#include <Wire.h>

// An anemometer for the Arduino
int LogInterval;
int SampInterval=2; 	//Number of seconds in the LogInterval

#define WAIT_TO_START    0 // Wait for serial input in setup()

const byte nsamp=10;

// the digital pins that connect to the LEDs
//#define redLEDpin 2
#define greenLEDpin 3
#define WindPin 4

// Wind variables
bool SeeHigh=false;
int CntRPM=0;
unsigned long CntPeriod=0;
float RPM=0.0;
float AvPeriod=0.0;
float MPH=0.0;
float Twc=0.0;	//Wind chill Temperature

// Timing counters
unsigned long StartTime;	//Log Interval counter
unsigned long StartPeriod=0;	// Period Start
unsigned long timeSense=0;

// Analog pins
int ThermPin=2;		//MPC9701 thermal sensor


unsigned int ThermValue = 0;  // value coming from thermal sensor
int ThermValueAvg;
float mVout;	// Temp voltage in mV	
float TempC=0.0;
float TempF=0.0;

void error(char *str)
{
  Serial.print("error: ");
  Serial.println(str);
  while(1);
}

void setup(void)
{
  analogReference(EXTERNAL); //3.3V connected to AREF thru 3.3K AREF=3VDC
  int dummy=analogRead(ThermPin); //discard first analogRead
  Serial.begin(9600);
  Serial.println();
  pinMode(WindPin,INPUT);
  
	#if WAIT_TO_START
		Serial.println("Type any character to start");
		while (!Serial.available());
	#endif //WAIT_TO_START

	Serial.println("TempF, RPM, MPH, Twc");

	LogInterval=SampInterval*1000;	 //  sec between entries
	StartTime=millis(); //Initialize StartTime
}

void loop(void)
{
	// Wind calculator Look for High
	if (digitalRead(WindPin)==HIGH)
    {
		SeeHigh=true;
		digitalWrite(greenLEDpin, LOW);
	}
	//Look for Low thus a High to Low transistion
	else if (SeeHigh==true)
	{
		if (StartPeriod != 0)	//Not first sampleFrt
		{
		  // Number of milliseconds in a revolution, added together
		  CntPeriod=CntPeriod+millis()-StartPeriod;
		}	
		StartPeriod=millis();
		//Increment counter
		CntRPM++;
		SeeHigh=false;
		digitalWrite(greenLEDpin,HIGH);
	}
   
	if ((millis()-StartTime)>long(LogInterval))  // Log Interval has passed
	{
		// Do Wind calculations for LogInterval
		// RPM is calculated, Period is averaged
		RPM=CntRPM*(60.0/SampInterval);
		AvPeriod=CntPeriod/(CntRPM-1);
		MPH=RPM*.054; //Estimate
		// Get temperature readings Average nsamps
		for (byte j=0;j<nsamp;j++)
		{    
			ThermValue += analogRead(ThermPin);    
		}    
		ThermValueAvg=ThermValue/nsamp;

		mVout=(float) ThermValueAvg*3000.0/1023.0; //3.0V = 3000mV
		//TempC=(mVout-400.0)/19.5; //Ta = (Vout-400mV)/19.5mV //Original
		//TempC=(mVout-580.0)/19.5; //Ta = (Vout-400mV)/19.5mV  //Modified
		TempC=(mVout-490.0)/19.5; //Ta = (Vout-400mV)/19.5mV  //Modified
		TempF=TempC*(9.0/5.0)+32; 
		
		//Wind Chill formula Twc=35.74+0.6215Tf-35.75MPH 0.16 +0.4275TfMPH 0.16
		//double pow (double base, double exponent )
		if ((TempF <50.0) && (MPH > 3.0))
		{
			Twc=35.74+0.6215*TempF-35.75*pow(MPH,0.16)+0.4275*TempF*pow(MPH,0.16);
		}
		else
		{
			Twc=TempF;
		}
		ThermValue=0; //Clear Buffers

		Serial.print("$, ");
		Serial.print(TempF,1);
		Serial.print(", ");
		Serial.print(RPM,1);
		Serial.print(", ");    
		Serial.print(MPH,1);
		Serial.print(", ");    
		Serial.print(Twc,1);
	
		StartTime=millis();
		Serial.println();

		StartTime = millis();	//Get StartTime for next LogInterval
		// Clear variables for next LogInterval
		digitalWrite(greenLEDpin, LOW);
		SeeHigh=false;
		CntRPM=0;
		AvPeriod=0.0;
	}
}